@@ -22,33 +22,24 @@
 #' @param lambda          A user-specified sequence of lambda values. By default, a sequence of values of length nlambda is computed, equally spaced on the log scale.
 #' @param eps             Convergence threshold. The algorithm iterates until the RMSD for the change in linear predictors for each coefficient is less than eps. Default is \code{1e-4}.
 #' @param max_iter        Maximum number of iterations (total across entire path). Default is 10000.
-#' @param convex          (future idea; not yet incorporated) Calculate index for which objective function ceases to be locally convex? Default is TRUE.
-#' @param dfmax           (future idea; not yet incorporated) Upper bound for the number of nonzero coefficients. Default is no upper bound. However, for large data sets, computational burden may be heavy for models with a large number of nonzero coefficients.
 #' @param init            Initial values for coefficients. Default is 0 for all columns of X.
 #' @param warn            Return warning messages for failures to converge and model saturation? Default is TRUE.
 #' @param type            A character argument indicating what should be returned from predict.plmm(). If type == 'lp', predictions are
 #'                        based on the linear predictor, X beta. If type == 'blup', predictions are based on the sum of the linear predictor
 #'                        and the estimated random effect (BLUP). Defaults to 'blup', as this has shown to be a superior prediction method
 #'                        in many applications.
-#' @param cluster         cv_plmm() can be run in parallel across a cluster using the parallel package. The cluster must be set up in
-#'                        advance using parallel::makeCluster(). The cluster must then be passed to cv_plmm().
+#' @param cluster         Option for **in-memory data only**: cv_plmm() can be run in parallel across a cluster using the parallel package.
+#'                        The cluster must be set up in advance using parallel::makeCluster(). The cluster must then be passed to cv_plmm().
+#'                        **Note**: this option is not yet implemented for filebacked data.
 #' @param nfolds          The number of cross-validation folds. Default is 5.
 #' @param fold            Which fold each observation belongs to. By default, the observations are randomly assigned.
 #' @param seed            You may set the seed of the random number generator in order to obtain reproducible results.
-#' @param returnY         Should cv_plmm() return the linear predictors from the cross-validation folds? Default is FALSE; if TRUE,
-#'                        this will return a matrix in which the element for row i, column j is the fitted value for observation i from
-#'                        the fold in which observation i was excluded from the fit, at the jth value of lambda.
-#' @param returnBiasDetails Logical: should the cross-validation bias (numeric value) and loss (n x p matrix) be returned? Defaults to FALSE.
 #' @param trace           If set to TRUE, inform the user of progress by announcing the beginning of each CV fold. Default is FALSE.
 #' @param save_rds        Optional: if a filepath and name *without* the '.rds' suffix is specified (e.g., `save_rds = "~/dir/my_results"`), then the model results are saved to the provided location (e.g., "~/dir/my_results.rds").
 #'                        Defaults to NULL, which does not save the result.
-#' @param save_fold_res   Optional: a logical value indicating whether the results (loss and predicted values) from each CV fold should be saved?
-#'                        If TRUE, then two '.rds' files will be saved ('loss' and 'yhat') will be created in the same directory as 'save_rds'.
-#'                        Both files will be updated after each fold is done.
-#'                        Defaults to FALSE.
+#'                        **Note**: Along with the model results, two '.rds' files ('loss' and 'yhat') will be created in the same directory as 'save_rds'.
+#'                        These files contain the loss and predicted outcome values in each fold; both files will be updated during after prediction within each fold.
 #' @param return_fit      Optional: a logical value indicating whether the fitted model should be returned as a `plmm` object in the current (assumed interactive) session. Defaults to TRUE.
-#' @param compact_save    Optional: if TRUE, three separate .rds files will saved: one with the 'beta_vals', one with 'K', and one with everything else (see below).
-#'                        Defaults to FALSE. **Note**: you must specify `save_rds` for this argument to be called.
 #' @param ...             Additional arguments to `plmm_fit`
 #'
 #' @returns a list with 12 items:
@@ -73,7 +64,7 @@
 #'
 #' @examples
 #' admix_design <- create_design(X = admix$X, y = admix$y)
-#' cv_fit <- cv_plmm(design = admix_design, return_fit = TRUE)
+#' cv_fit <- cv_plmm(design = admix_design)
 #' print(summary(cv_fit))
 #' plot(cv_fit)
 #'
@@ -95,43 +86,25 @@ cv_plmm <- function(design,
                     lambda,
                     eps = 1e-04,
                     max_iter = 10000,
-                    convex = TRUE,
-                    dfmax = NULL,
                     warn = TRUE,
                     init = NULL,
                     cluster,
                     nfolds=5,
                     seed,
                     fold = NULL,
-                    returnY=FALSE,
-                    returnBiasDetails = FALSE,
                     trace=FALSE,
                     save_rds = NULL,
-                    save_fold_res = FALSE,
                     return_fit = TRUE,
-                    compact_save = FALSE,
                     ...) {
 
   # check filepaths for saving results ------------------------------
-  if (save_fold_res & is.null(save_rds)) {
-    stop("You have set 'save_fold_res = TRUE', but no argument was supplied to 'save_rds'.
-         \nPlease specify a filepath (as a string) to 'save_rds'")
-  }
-
-  if (compact_save & is.null(save_rds)) {
-    stop("You have set 'compact_save = TRUE', but no argument was supplied to 'save_rds'.
-          \nPlease specify a filepath (as a string) to 'save_rds'")
-  }
-
   if (!is.null(save_rds)) {
     save_rds <- check_for_file_extension(save_rds)
     # ^^ internally, we need to take off the extension from the file name
-  }
 
-  # start the log -----------------------
-  logfile <- create_log(outfile = ifelse(!is.null(save_rds),
-                                         save_rds,
-                                         "./cv-plmm"))
+    # start the log
+    logfile <- create_log(outfile = save_rds)
+  }
 
   # create a design if needed -------------
   if (inherits(design, "data.frame") | inherits(design, 'matrix')) {
@@ -158,21 +131,21 @@ cv_plmm <- function(design,
                               eta_star = eta_star,
                               penalty = penalty,
                               init = init,
-                              dfmax = dfmax,
                               gamma = gamma,
                               alpha = alpha,
                               trace = trace,
                               ...)
 
-  cat("\nInput data passed all checks at ",
-      pretty_time(),
-      file = logfile, append = TRUE)
+  if (!is.null(save_rds)){
+    cat("\nInput data passed all checks at ",
+        pretty_time(),
+        file = logfile, append = TRUE)
+  }
 
   # prep  ------------------------
   prep_args <- c(list(std_X = checked_data$std_X,
                       std_X_n = checked_data$std_X_n,
                       std_X_p = checked_data$std_X_p,
-                      genomic = checked_data$genomic,
                       n = checked_data$n,
                       p = checked_data$p,
                       centered_y = checked_data$centered_y,
@@ -183,9 +156,12 @@ cv_plmm <- function(design,
                       trace = trace))
 
   prep <- do.call('plmm_prep', prep_args)
-  cat("\nEigendecomposition finished at ",
-      pretty_time(),
-      file = logfile, append = TRUE)
+
+  if (!is.null(save_rds)){
+    cat("\nEigendecomposition finished at ",
+        pretty_time(),
+        file = logfile, append = TRUE)
+  }
 
   # full model fit ----------------------------------
   fit_args <- c(list(
@@ -201,9 +177,7 @@ cv_plmm <- function(design,
     nlambda = nlambda,
     max_iter = max_iter,
     eps = eps,
-    warn = warn,
-    convex = convex,
-    dfmax = dfmax))
+    warn = warn))
 
   if (!missing(lambda_min)){
     fit_args$lambda_min <- lambda_min
@@ -211,33 +185,29 @@ cv_plmm <- function(design,
 
   fit <- do.call('plmm_fit', fit_args)
 
-  cat("\nFull model fit finished at",
-      pretty_time(),
-      file = logfile, append = TRUE)
+  if (!is.null(save_rds)){
+    cat("\nFull model fit finished at",
+        pretty_time(),
+        file = logfile, append = TRUE)
+  }
 
   fit_to_return <- plmm_format(fit = fit,
                                p = checked_data$p,
                                std_X_details = checked_data$std_X_details,
-                               fbm_flag = checked_data$fbm_flag)
+                               fbm_flag = checked_data$fbm_flag,
+                               plink_flag = checked_data$plink_flag)
 
-  cat("\nFormatting for full model finished at",
-      pretty_time(),
-      file = logfile, append = TRUE)
+  if (!is.null(save_rds)){
+    cat("\nFormatting for full model finished at",
+        pretty_time(),
+        file = logfile, append = TRUE)
+  }
 
   if (!is.null(save_rds)) {
-    if (compact_save) {
-      # go ahead and save the most important results from the full model fit
-
-      saveRDS(fit_to_return$beta_vals, paste0(save_rds, "_coefficients.rds"))
-      cat("Coefficients (estimated beta values) saved to:", paste0(save_rds, "_coefficients.rds"), "at",
-          pretty_time(),
-          file = logfile, append = TRUE)
-
-      saveRDS(fit_to_return$K, paste0(save_rds, "_K.rds"))
-      cat("K (eigendecomposition) saved to:", paste0(save_rds, "_K.rds"), "at",
-          pretty_time(),
-          file = logfile, append = TRUE)
-    }
+    saveRDS(fit_to_return, paste0(save_rds, ".rds"))
+    cat("Results saved to:", paste0(save_rds, ".rds"), "at",
+        pretty_time(),
+        file = logfile, append = TRUE)
   }
   gc()
 
@@ -245,7 +215,8 @@ cv_plmm <- function(design,
   cv_args <- fit_args
   cv_args$warn <- FALSE
   cv_args$lambda <- fit$lambda
-  cv_args$non_genomic <- checked_data$non_genomic
+  cv_args$eta_star <- fit$eta # note: here we use the same eta estimate in each fold of CV
+  cv_args$plink_flag <- checked_data$plink_flag
 
   estimated_Sigma <- NULL
   if (type == 'blup') {
@@ -256,19 +227,11 @@ cv_plmm <- function(design,
   n <- checked_data$std_X_n
   E <- Y <- matrix(NA, nrow=n, ncol=length(fit$lambda))
 
-
   # set up folds for cross validation
   if (!missing(seed)) {
     original_seed <- .GlobalEnv$.Random.seed
     on.exit(.GlobalEnv$.Random.seed <- original_seed)
     set.seed(seed)
-  } else {
-    # TODO: determine how, if at all, the random seed should be documented
-    # cat("Random seed:",
-    #     .GlobalEnv$.Random.seed[1],
-    #     "\n",
-    #     file = logfile,
-    #     append = TRUE)
   }
 
   sde <- sqrt(.Machine$double.eps)
@@ -286,39 +249,65 @@ cv_plmm <- function(design,
 
   # set up cluster if user-specified ---------------------------------------
   if (!missing(cluster)) {
+
+    # error check: parallelization is not yet implemented for filebacked data
+    if (checked_data$fbm_flag) stop("Parallelization is not yet implemented for filebacked data.
+                                    You cannot specify a 'cluster' argument if data are stored filebacked.\n.")
+
+    # check type of 'cluster' argument
     if (!inherits(cluster, "cluster")) stop("cluster is not of class 'cluster'; see ?makeCluster", call.=FALSE)
-    parallel::clusterExport(cluster, c("design", "K", "fold", "type", "cv_args", "estimated_Sigma"), envir=environment())
+
+    # check if variables are defined
+    if (!exists("fold") || !exists("type") || !exists("cv_args")) {
+      stop("One or more required variables (fold, type, cv_args) are not defined", call.=FALSE)
+    }
+
+    parallel::clusterExport(cl = cluster,
+                            varlist = c("fold", "type", "cv_args"),
+                            envir = environment())
     parallel::clusterCall(cluster, function() library(plmmr))
-    fold.results <- parallel::parLapply(cl=cluster, X=1:max(fold), fun=cvf,
+    fold_results <- parallel::parLapply(cl=cluster,
+                                        X=1:max(fold),
+                                        fun=cvf,
                                         design = design,
-                                        fold=fold, type=type, cv_args=cv_args,
-                                        estimated_Sigma = estimated_Sigma)
+                                        fold=fold,
+                                        type=type,
+                                        cv_args=cv_args)
+
+    # stop the cluster when done
+    parallel::stopCluster(cluster)
+
   }
 
   # carry out CV -------------------------------------
   if (trace) cat("\nStarting cross validation\n")
-  cat("\nCross validation started at: ",
-      pretty_time(),
-      file = logfile,
-      append = TRUE)
+
+  if (!is.null(save_rds)){
+    cat("\nCross validation started at: ",
+        pretty_time(),
+        file = logfile,
+        append = TRUE)
+  }
 
   # set up progress bar -- this can take a while
   if(trace){pb <- utils::txtProgressBar(min = 0, max = nfolds, style = 3)}
   for (i in 1:nfolds) {
     # case 1: user-specified cluster
     if (!missing(cluster)) {
-      res <- fold.results[[i]] # refers to lines from above
+      res <- fold_results[[i]] # refers to lines from above
       if (trace) {utils::setTxtProgressBar(pb, i)}
 
     } else {
-      # case 2: cluster NOT user specified (this is the typical use case)
-      cat("Started fold", i, "at", pretty_time(),
-          file = logfile, append = TRUE)
+      # case 2: cluster NOT user specified
+      if (!is.null(save_rds)){
+        cat("Started fold", i, "at", pretty_time(),
+            file = logfile, append = TRUE)
+      }
+
       res <- cvf(i = i,
                  fold = fold,
                  type = type,
-                 cv_args = cv_args,
-                 estimated_Sigma = estimated_Sigma)
+                 cv_args = cv_args)
       if (trace) {utils::setTxtProgressBar(pb, i)}
 
     }
@@ -333,7 +322,7 @@ cv_plmm <- function(design,
     }
     Y[fold==i, 1:res$nl] <- res$yhat
 
-    if (save_fold_res) {
+    if (!is.null(save_rds)) {
       saveRDS(E, paste0(save_rds, "_loss.rds"))
       cat("Loss saved to:", paste0(save_rds, "_loss.rds"), "at", pretty_time(),
           file = logfile, append = TRUE)
@@ -362,6 +351,7 @@ cv_plmm <- function(design,
   u.se <- cve[min] + cvse[min]
   within1se <- which(cve >= l.se & cve <= u.se)
   min1se <- which.max(lambda %in% lambda[within1se])
+
   # bias correction
   e <- sapply(1:nfolds, function(i) apply(E[fold==i, , drop=FALSE], 2, mean))
   Bias <- mean(e[min,] - apply(e, 2, min))
@@ -376,64 +366,29 @@ cv_plmm <- function(design,
               lambda_min=lambda[min],
               min1se = min1se,
               lambda.1se = lambda[min1se],
-              null.dev=mean(plmm_loss(checked_data$y, rep(mean(checked_data$y), n))))
-
-  if (returnY) val$Y <- Y
-  if (returnBiasDetails){
-    val$Bias <- Bias
-    val$Loss <- E
-  }
+              null.dev=mean(plmm_loss(checked_data$y, rep(mean(checked_data$y), n))),
+              Y = Y,
+              bias = Bias,
+              loss = E)
 
   if (type == "blup"){
     val$estimated_Sigma = estimated_Sigma
   }
 
-  # handle output
+  # save output
   if (!is.null(save_rds)){
-    if (compact_save) {
-      # save the rest of the output across multiple files
-      saveRDS(val[c(1:5, 7:11)], paste0(save_rds, "_cv_details.rds"))
-      cat("CV details (CVE, fold assignments, etc) saved to:",
-          paste0(save_rds, "_cv_details.rds"),
-          "at",
-          pretty_time(),
-          file = logfile, append = TRUE)
-
-      saveRDS(estimated_Sigma, paste0(save_rds, "_estimated_Sigma.rds"))
-      cat("Estimated variance matrix saved to:", paste0(save_rds, "_estimated_Sigma.rds"),
-          "at", pretty_time(), file = logfile, append = TRUE)
-
-      saveRDS(fit_to_return$linear_predictors, paste0(save_rds, "_linear_predictors.rds"))
-      cat("Linear predictors (on rotated scale) saved to:", paste0(save_rds, "_linear_predictors.rds"), "at",
-          pretty_time(),
-          file = logfile, append = TRUE)
-
-      saveRDS(fit_to_return[c(2:3, 5:12)], paste0(save_rds, "_full_fit_details.rds"))
-      cat("All other results (loss, # of iterations, ...) saved to:", paste0(save_rds, "_full_fit_details.rds"), "at",
-          pretty_time(),
-          file = logfile, append = TRUE)
-
-    } else {
-      # save all output in one file (default)
-      saveRDS(val, paste0(save_rds, ".rds"))
-      cat("Results saved to:", paste0(save_rds, ".rds"), "at",
-          pretty_time(),
-          file = logfile, append = TRUE)
-    }
-
-
+    saveRDS(val, paste0(save_rds, ".rds"))
+    cat("Results saved to:", paste0(save_rds, ".rds"), "at",
+        pretty_time(),
+        file = logfile, append = TRUE)
   }
 
+  # create a failsafe -- if save_rds is NULL, make sure return_fit = TRUE
   if (is.null(save_rds) & !return_fit){
-    cat("\nYou accidentally left save_rds NULL while setting return_fit = FALSE;
-        to prevent you from losing your work, I am saving the output as cv_plmm_results.rds
-        in your current working directory (current folder).
-        \nNext time, make sure to specify your own filepath to the save_rds argument.")
+    cat("You accidentally left save_rds = NULL and return_fit = FALSE;
+        to prevent you from losing your work, plmm() is returning the output as if return_fit = TRUE")
 
-    rdsfile <- paste0(getwd(),"/cv_plmm_results.rds")
-    saveRDS(val, rdsfile)
-    cat("Results saved in", rdsfile, "at", pretty_time(),
-        file = logfile, append = TRUE)
+    return_fit <- TRUE
   }
 
   # release pointer
