@@ -79,12 +79,8 @@ plot.cv_plmm <- function(x, log.l=TRUE, type=c("cve", "rsq", "scale", "snr", "al
   suppressWarnings(graphics::arrows(x0=l[aind], x1=l[aind], y0=L[aind], y1=U[aind], code=3, angle=90, col="gray80", length=.05))
   graphics::points(l[ind], y[ind], col=col, pch=19, cex=.5)
   if (selected) {
-    if(inherits(x$fit, "list")){
-      n.s <- predict_within_cv(x$fit, lambda=x$lambda, type="nvars")
-    } else {
-      n.s <- predict(x$fit, lambda=x$lambda, type="nvars")
-    }
-    graphics::axis(3, at=l, labels=n.s, tick=FALSE, line=-0.5)
-    graphics::mtext("Variables selected", cex=0.8, line=1.5)
+    n.s <- predict(x$fit, lambda=x$lambda, type="nvars")
   }
-}
+  graphics::axis(3, at=l, labels=n.s, tick=FALSE, line=-0.5)
+  graphics::mtext("Variables selected", cex=0.8, line=1.5)
+}
\ No newline at end of file
