@@ -18,7 +18,7 @@
 #' print(summary(cv_fit))
 #'
 print.summary.cv_plmm <- function(x, digits, ...){
-  n <- nrow(x$fit$linear_predictors)
+  n <- nrow(x$fit$std_Xbeta)
   p <- nrow(x$fit$beta_vals)
   digits <- if (missing(digits)) digits <- c(2, 4, 2, 2, 3) else rep(digits, length.out=5)
   cat(x$fit$penalty, "-penalized model with n=", n, " and p=", p,"\n", sep="")
